@@ -1,10 +1,10 @@
 # FLSR.R - FLSR class and methods for S/R data
 
 # Author: FLR Team
-# Maintainer: Richard Hillary
+# Maintainer: Dorleta García, AZTI Tecnalia
 # Additions:
-# Last Change: 19 Dec 2005 23:44
-# $Id: FLSR.R,v 1.50.2.6 2005/12/20 08:16:00 iagoazti Exp $
+# Last Change: 28 mar 2006 09:20
+# $Id: FLSR.R,v 1.50.2.20 2006/03/28 07:19:58 iagoazti Exp $
 
 # Reference:
 # Notes:
@@ -61,100 +61,58 @@ setClass("FLSR",
 #remove(validFLSR)	
 # }}}
 
+validFLSR <- function(object) {
+
+	## FLQuant objects that must have same dimensions
+	dim <- dim(object@rec)
+
+	s <- list("rec", "ssb", "rechat", "residuals")
+	
+    for (i in s) 
+        if (!all(dim(slot(object, i)) == dim))
+			stop(paste("FLQuant dimensions wrong for", i))
+
+	return(TRUE)
+}
+
+
 ## FLSR()                   {{{
 FLSR <- function(rec="missing", ssb="missing", rec.age="missing", ...) {
 
+    if(missing(rec.age) && (is.FLStock(rec) || is.FLBiol(rec))) 
+        rec.age <- dims(m(rec))$min
+    
     # rec = FLQuant
     if (is.FLQuant(rec) && missing(ssb)) {
         
         sr <- new("FLSR")
 
-        slot(sr, "ssb") <- FLQuant(rec)
-        slot(sr, "rec") <- FLQuant(rec)
-        slot(sr, "rechat") <- FLQuant(rec)
-        slot(sr, "residuals") <- FLQuant(rec)
+        slot(sr, "ssb") <- FLQuant(dimnames = dimnames(rec))
+        slot(sr, "rec") <- FLQuant(dimnames = dimnames(rec))
+        slot(sr, "rechat") <- FLQuant(dimnames = dimnames(rec))
+        slot(sr, "residuals") <- FLQuant(dimnames = dimnames(rec))
     }
 
     # rec = FLQuant & ssb = FLQuant
     else if (is.FLQuant(rec) && is.FLQuant(ssb)) {
 
-        sr <- new("FLSR")
+            sr <- new("FLSR")
 
-        slot(sr, "ssb") <- ssb
-        slot(sr, "rec") <- rec
-        slot(sr, "rechat") <- FLQuant(rec)
-        slot(sr, "residuals") <- FLQuant(rec)
-    }
+            slot(sr, "ssb") <- ssb
+            slot(sr, "rec") <- rec
+            slot(sr, "rechat") <- FLQuant(dimnames = dimnames(rec))
+            slot(sr, "residuals") <- FLQuant(dimnames = dimnames(rec))
+        }
 
-    # rec = FLStock
-    else if (is.FLStock(rec)) {
+        # rec = FLStock
+        else if (is.FLStock(rec)) 
+                sr <- FLSRstock (rec, rec.age = rec.age) 
+             else if (is.FLBiol(rec))
+                sr <- FLSRbiol(rec, rec.age)
 
-    	sr <- new("FLSR")
-
-    	# recruitment delay set using minage
-		# from the FLStock object
-
-		if(missing(rec.age))
-			rec.age <- rec@range[['min']]
-    
-        if (sum(complete.cases(slot(rec, "stock.n")))  == 0 |
-            sum(complete.cases(slot(rec, "stock.wt")))  == 0 |
-            sum(complete.cases(slot(rec, "mat")))  == 0  | 
-	    sum(complete.cases(slot(rec, "harvest.spwn"))) == 0 | 
-	    sum(complete.cases(slot(rec, "m.spwn"))) == 0)
-                stop("stock must have 'stock.n', 'stock.wt', 'harvest.spwn', 'm.spwn' and 'mat'")
-
-        # calculate ssb and create FLSR object incorprating rec.age
-        
-	if(missing(rec.age)) {
-		.rec <- rec@stock.n[1,]
-	} else {
-		if(rec.age < rec@range[['min']])
-			stop("Supplied recruitment age less than minimum age class")
-			diffrec <- rec.age-rec@range[['min']]	
-			.rec <- rec@stock.n[1+diffrec,] 
-	}
-		 
-	if(units(slot(rec,"harvest")) != 'f' && units(slot(rec,"harvest")) != 'hr')
-		stop("Incorrect units specified in harvest slot of FLStock object")
-	if(units(slot(rec, "harvest")) == 'f')
-		.ssb <- apply(slot(rec, "stock.n") * exp(-harvest(rec, "f") *
-       		slot(rec,"harvest.spwn")-slot(rec, "m")*
-		slot(rec, "m.spwn"))*slot(rec, "stock.wt")*
-		slot(rec, "mat"), 2:5, sum)
-	if(units(slot(rec, "harvest")) == 'hr')
-		.ssb <- apply(slot(rec, "stock.n") * (1 - slot(rec, "harvest") * slot(rec, "harvest.spwn")) *
-		exp(-slot(rec, "m") * slot(rec, "m.spwn")) * slot(rec, "mat") * slot(rec, "stock.wt"), 2:5, sum)	
-
-
-		# now alter the stock and recruitment 
-		# vectors to factor in the recruitement age
-	
-	if((length(.rec[,,1,1,1])-1) <= rec.age)
-		stop("FLStock recruitment data set too short")
-		
-	.rec <- .rec[,(1+rec.age):length(.rec[,,1,1,1]),,,]
-	.ssb <- .ssb[,1:(length(.ssb[,,1,1,1])-rec.age),,,]
-	if(length(.rec[,,1,1,1]) != length(.ssb[,,1,1,1]))
-		stop("FLStock stock-recruit data not of same length")
-
-	# create the FLSR object
-
-	sr <- new("FLSR", rec=.rec,ssb=.ssb) 
-        units(slot(sr, "rec")) <- units(slot(rec, "stock.n"))
-	units(slot(sr, "ssb")) <- units(slot(rec, "stock.wt")) 
-
-        slot(sr, "rechat") <- FLQuant(slot(sr, "rec"))
-        units(slot(sr, "rechat")) <- units(slot(sr, "rec"))
-        
-        slot(sr, "residuals") <- FLQuant(slot(sr, "rec"))
-	units(slot(sr, "residuals")) <- units(slot(sr, "rec")) 
-
-    }
-
-    else {
-        sr <- new("FLSR")
-    }
+                  else 
+                    sr <- new("FLSR")
+                  
     
     # add other specified slots
     args <- list(...)
@@ -206,52 +164,38 @@ if (!isGeneric("predict")) {
 setMethod("predict", signature(object="FLSR"),
 	function(object, ssb="missing",...){
 
-      		if (missing(ssb))
-         		ssb <- object@ssb
+        if (missing(ssb))
+            ssb <- object@ssb
 
 		# ricker
-		PredictRicker <- function(object)
+		PredictRicker <- function(object, ssb)
 			object@params[,"alpha"] * ssb * exp(-object@params[,"beta"] * ssb)
 
 		# bevholt
-		Predictbevholt <- function(object)
+		Predictbevholt <- function(object, ssb)
 			object@params[,"alpha"] * ssb / (ssb + object@params[,"beta"])
 
 		# segreg
-		Predictsegreg <- function(object) {
-			.retval <- vector(length=length(ssb))
-			for(i in 1:length(ssb)) {
-				if(ssb[i] <= object@params[,"beta"])
-					.retval[i] <- object@params[,"alpha"]*ssb[i]
-				if(ssb[i] > object@params[,"beta"])
-					.retval[i] <- object@params[,"alpha"]*object@params[,"beta"]
-			}
-			return(.retval)
-		}
+		Predictsegreg <- function(object, ssb) 
+            ifelse(ssb <= object@params[,"beta"], object@params[,"alpha"]*ssb, 
+                    object@params[,"alpha"]*object@params[,"beta"])
 
-		Predictqhstk <- function(object) {
-			.retval <- vector(length=length(ssb))
-			for(i in 1:length(ssb)) {
-				if(ssb[i] <= object@params[[,"beta"]]*(1-object@params[[,"rho"]]))
-					.retval[i] <- object@params[[,"alpha"]]*ssb[i]
-				if(ssb[i] > object@params[[,"beta"]]*(1-object@params[[,"rho"]]) && ssb[i] < object@params[[,"beta"]]*(1+object@params[[,"rho"]]))
-					.retval[i] <- object@params[[,"alpha"]]*(ssb[i]-(ssb[i]-object@params[[,"beta"]]*(1-object@params[[,"rho"]]))*(ssb[i]-object@params[[,"beta"]]*(1-object@params[[,"rho"]]))/(as.double(4)*object@params[[,"beta"]]*object@params[[,"rho"]]))
-				if(ssb[i] >= object@params[[,"beta"]]*(1+object@params[[,"rho"]]))
-					.retval[i] <- object@params[[,"alpha"]]*object@params[[,"beta"]]
+        # qhstk
+		Predictqhstk <- function(object, ssb) 
+			ifelse(ssb <= object@params[,"beta"]*(1-object@params[,"rho"]),
+			     object@params[,"alpha"]*ssb, ifelse(ssb < object@params[,"beta"]*(1+object@params[,"rho"]),
+                 object@params[,"alpha"]*(ssb-(ssb-object@params[,"beta"]*(1-object@params[,"rho"]))*(ssb-object@params[,"beta"]*(1-object@params[,"rho"]))/(as.double(4)*object@params[,"beta"]*object@params[,"rho"])), 
+			     object@params[,"alpha"]*object@params[,"beta"]))
 
-			}
-			return(.retval)
-		}
-		 
 		# Main part of the predict method
 		if(length(grep('ricker', object@model)) != 0) 
-			res <- PredictRicker(object)
+			res <- PredictRicker(object, ssb)
 		if(length(grep('bevholt', object@model)) != 0) 
-			res <- Predictbevholt(object) 
+			res <- Predictbevholt(object, ssb) 
 		if(length(grep('segreg', object@model)) != 0) 
-			res <- Predictsegreg(object) 
+			res <- Predictsegreg(object, ssb) 
 		if(length(grep('qhstk', object@model)) != 0) 
-			res <- Predictqhstk(object) 
+			res <- Predictqhstk(object, ssb) 
 
 		return(res)
 	}
@@ -260,7 +204,7 @@ setMethod("predict", signature(object="FLSR"),
 ## plot :: FLSR                 {{{
 setMethod("plot", signature(x="FLSR", y="missing"),
     function(x, y, type=c("all", "SR", "QQ", "Res.AR1", "Res.t", "Res.S", "Res.R"),
-             cols=c(1, 2, 8), ...) {
+             cols=c(1, 2, 8), xlab = "SSB", ...) {
 	        
 		if(is.na(x@aic)) {
 
@@ -283,16 +227,16 @@ setMethod("plot", signature(x="FLSR", y="missing"),
 			lines(ssb, as.vector(predict(x,ssb)),col="red") 
 
 		} else {
-        	ssb <-x@ssb[ 1,,,,,drop=TRUE]
-			r   <-x@rec[   1,,,,,drop=TRUE]
+        	ssb <-x@ssb[1,,,,,drop=TRUE]
+			r   <-x@rec[1,,,,,drop=TRUE]
         	rhat<-x@rechat[1,,,,,drop=TRUE]
         	year<-as.integer(dimnames(x@rec)$year)
 
         	res <- log(r / rhat)
         		  
 		#S-R plot with fitted curve
-		SRPlot <- function(ssb, r) {
-			plot(ssb, r, xlim=c(0, max(ssb)+1), ylim=c(0, max(r)+1), xlab="SSB",
+		SRPlot <- function(ssb, r, xlab=xlab) {
+			plot(ssb, r, xlim=c(0, max(ssb)+1), ylim=c(0, max(r)+1), xlab=xlab,
                 ylab="Recruits", main="Stock Recruit Plot", col=cols[1])
 			ssb <- 0:100 * max(ssb) * 0.012
 			lines(ssb, as.vector(predict(x,ssb)), col=cols[2])
@@ -321,9 +265,9 @@ setMethod("plot", signature(x="FLSR", y="missing"),
 		}
 
 		#Residuals plotted against SSB
-		SRPlotResidualsByX <- function(ssb,res){
-			plot(ssb, res, xlab="SSB", ylab="Residuals", main="Residuals by SSB",
-                col=cols[1])
+		SRPlotResidualsByX <- function(ssb,res, xlab=xlab){
+			plot(ssb, res, xlab=xlab, ylab="Residuals", 
+                main=paste("Residuals by", xlab), col=cols[1])
 			lines(lowess(ssb, res), col=cols[2])
 			lines(c(min(ssb), max(ssb)), c(0, 0), col=cols[3], lty=2)
 			invisible()
@@ -351,12 +295,12 @@ setMethod("plot", signature(x="FLSR", y="missing"),
 
 
 		#All plots arranged on a single page
-		SRPlots <- function(ssb,r,rhat,year,res){
+		SRPlots <- function(ssb,r,rhat,year,res, xlab){
 			par(mfrow=c(3, 2))
-			SRPlot(ssb,r)
+			SRPlot(ssb,r, xlab)
 			SRPlotResidualsByYear(year,res)
 			SRPlotAR1(res)
-			SRPlotResidualsByX(ssb,res)
+			SRPlotResidualsByX(ssb,res, xlab)
 			SRPlotQQ(res)
 			SRPlotResidualsByYHat(rhat,res)
 			invisible()
@@ -367,12 +311,12 @@ setMethod("plot", signature(x="FLSR", y="missing"),
 		on.exit(par(opar))
 		par(mfrow=c(1, 1))
 		switch(as.character(type[1]),
-			"all"=SRPlots(ssb,r,rhat,year,res),
-			"SR"=SRPlot(ssb,r),
+			"all"=SRPlots(ssb,r,rhat,year,res, xlab),
+			"SR"=SRPlot(ssb,r, xlab),
 			"QQ"=SRPlotQQ(res),
 			"Res.AR1"=SRPlotAR1(res),
 			"Res.t"=SRPlotResidualsByYear(year,res),
-			"Res.S"=SRPlotResidualsByX(ssb,res),
+			"Res.S"=SRPlotResidualsByX(ssb,res, xlab),
 			"Res.R"=SRPlotResidualsByYHat(rhat,res),
 			stop("type must be 'all', 'SR', 'QQ', 'Res.AR1', 'Res.t', 'Res.S' or 'Res.R'!"))
         		invisible()
@@ -380,7 +324,7 @@ setMethod("plot", signature(x="FLSR", y="missing"),
 	}
 ) # }}}
 
-## residuals		{{{
+## params		{{{
 if (!isGeneric("params")) {
 	setGeneric("params", function(object, ...){
 		value  <-  standardGeneric("params")
@@ -407,20 +351,19 @@ if (!isGeneric("params<-")) {
 	})
 }
 
-#setMethod("params<-", signature(object="FLSR", value="matrix"),
-#    function(object, value) {
-#        object@params <- matrix(value, ncol=3, dimnames=list(1:nrow(value), c('alpha','beta','rho')))
-#        return(object)
-#    }
-#)
+setMethod("params<-", signature(object="FLSR", value="matrix"),
+	function(object, value) {
+		object@params <- matrix(value, ncol=3, dimnames=list(1:nrow(value), c('alpha','beta','rho')))
+		return(object)
+	}
+)
 
 setMethod("params<-", signature(object="FLSR", value="vector"),
-	function(object, param=NULL, value) {
-		if(length(value) != nrow(object@params))
-			stop("Length of input vector does not match that of the params slot")
-		object@params[,param] <- value
-		if(nrow(object@params) == 1)
-			dimnames(object@params)[[1]] <- ""
+	function(object, param=1:3, value) {
+		if (missing(param))
+			object@params <- matrix(value, ncol=3, dimnames=list(1, c('alpha','beta','rho')))
+		else
+			object@params[,param] <- value
 		return(object)
 	}
 ) # }}}
@@ -475,7 +418,7 @@ srlkhd <- function(x) {
 		stop("FLSR object has no model")
 		
 	if(x@model == "bevholt" | x@model == "ricker" | x@model == "segreg") {
-		if(is.na(x@params[,'alpha',]) | is.na(x@params[,'beta']))
+		if(is.na(x@params[,'alpha']) | is.na(x@params[,'beta']))
 			stop("Initial parameter estimates missing")
 	} else {
 		if(is.na(x@params[,'alpha']) | is.na(x@params[,'beta']) | is.na(x@params[,'rho']))
@@ -514,11 +457,12 @@ ricker <- function(rec, ssb, params = "missing") {
 	# create results list - ToDo
 
 	# input checks
-	if (!is.vector(rec) | !is.vector(ssb))
-		return(paste("Inputs not vectors"))
+	if(!is.FLQuant(rec) || !is.FLQuant(ssb)) stop("'rec' and 'ssb' must be 'FLQuant' objects")
+	
+	rec <- dimSums(rec)
+	ssb <- dimSums(ssb)
 
 	# create the output list
-
 	res <- list(rechat=as.vector(NA),
 		    residuals=as.vector(NA),
 			params = matrix(NA, ncol=3, dimnames=list("", c('alpha','beta','rho'))),
@@ -530,7 +474,6 @@ ricker <- function(rec, ssb, params = "missing") {
 		    aic=as.double(NA)) 
 
 	# perform the MLE
-
 	T <- length(ssb)
 	x <- ssb
 	y <- log(rec/ssb)
@@ -546,29 +489,24 @@ ricker <- function(rec, ssb, params = "missing") {
 	.alpha <- exp(.alpha)
 
 	#  AIC calculation
-
 	.rhat <- .alpha*ssb*exp(-.beta*ssb)
 	.var <- var(log(rec)-log(.rhat))
-	logL <- 0
-	for(i in 1:length(rec)) {
-		logL <- logL + dnorm(log(rec[i]),log(.rhat[i]),sqrt(.var),TRUE)
-	}
+	logL <- sum(dnorm(log(rec),log(.rhat),sqrt(.var),TRUE))
+	
 	.aic <- -2*logL+3
 	
 	# calculate the variance and the autocorrelation-corrected variance
-
 	res$rechat <- .rhat
 	res$residuals <- log(rec)-log(.rhat)
 	res$params[,'alpha'] <- .alpha
 	res$params[,'beta'] <- .beta
 	res$var <- .var
-	acorr <- acf(res$residuals,plot=FALSE)$acf[2]
+	acorr <- acf(c(res$residuals),plot=FALSE)$acf[2]
 	res$varacorr <- .var*(1-acorr*acorr)
 
 	# calculate the covariance matrix: direct estimate 
 	# from inverse of the hessian of the log(lkhd) at the 
 	# MLE
-
 	covar <- matrix(nrow=2,ncol=2)
 	covar[1,1] <- length(rec)/(.alpha*.alpha*.var)
 	covar[1,2] <- -sum(ssb)/(.alpha*.var)
@@ -576,7 +514,6 @@ ricker <- function(rec, ssb, params = "missing") {
 	covar[2,2] <- sum(ssb*ssb)/.var
 
 	# invert this to get the variance-covariance matrix
-
 	covar <- solve(covar)
 	res$covar[1:2,1:2] <- covar[1:2,1:2] 
 	res$se['alpha'] <- sqrt(covar[1,1])
@@ -597,10 +534,10 @@ ricker <- function(rec, ssb, params = "missing") {
 bevholt <- function(rec, ssb, params = "missing") {
 
 	# input checks
+	if(!is.FLQuant(rec) || !is.FLQuant(ssb)) stop("'rec' and 'ssb' must be 'FLQuant' objects") 
+	rec <- dimSums(rec)
+	ssb <- dimSums(ssb)
 	
-	if (!is.vector(rec) | !is.vector(ssb))
-		return(paste("Inputs not vectors"))
-
 	# create the output list
 
 	res <- list(rechat=as.vector(NA),
@@ -613,9 +550,6 @@ bevholt <- function(rec, ssb, params = "missing") {
 		    varacorr=as.double(NA),
 		    aic=as.double(NA))
 
-	# create the predicted recruitment vector
-
-	.rhat <- vector(length=length(rec))
 
 	# perform the MLE using the optim() algorithm
 
@@ -626,43 +560,36 @@ bevholt <- function(rec, ssb, params = "missing") {
 		.beta <- x[2]
 		.sigma <- x[3]
 		.rhat <- x[1]*ssb/(x[2]+ssb)
-		logL <- 0
-		for(i in 1:length(rec)) {
-			logL <- logL - dnorm(log(rec[i]),log(.rhat[i]),sqrt(.sigma),TRUE)
-		}
+		logL <- - sum(dnorm(log(rec),log(.rhat),sqrt(.sigma),TRUE))
+		return(logL)
+
 	}
 
 	# setting up the initial guess and the bounds... dirty 
 	# version: alpha = max(rec)+0.1*(max(rec)-min(rec), beta = 0.5*min(ssb)
-        param <- numeric(3)
-    	lower <- numeric(3)
-	upper <- numeric(3)
+    param <- numeric(3)
 
 	if(!missing(params)) {
 		if(is.na(params[[1]]) | is.na(params[[2]])) { 
-    			param[1] <- max(rec)+0.1*(max(rec)-min(rec))
+        param[1] <- max(rec)+0.1*(max(rec)-min(rec))
 			param[2] <- 0.5*min(ssb)
 			.rhat <- param[1]*ssb/(param[2]+ssb)
-			param[3] <- var(log(rec/.rhat)) 
+			param[3] <- var(log(rec/.rhat), y= NULL, na.rm = TRUE) 
 		} else {
 			param[1] <- params[[1]]
 			param[2] <- params[[2]]
 			.rhat <- param[1]*ssb/(param[2]+ssb)
-			param[3] <- var(log(rec/.rhat))
+			param[3] <- var(log(rec/.rhat), y= NULL, na.rm = TRUE)
 		}
 	} else {
 		param[1] <- max(rec)+0.1*(max(rec)-min(rec))
 		param[2] <- 0.5*min(ssb)
 		.rhat <- param[1]*ssb/(param[2]+ssb)
-		param[3] <- var(log(rec/.rhat)) 	
+		param[3] <- var(log(rec/.rhat), y= NULL, na.rm = TRUE) 	
 	}
 		
-    	lower[1] <- 0.0
-    	upper[1] <- Inf
-    	lower[2] <- 0.001
-    	upper[2] <- Inf
-    	lower[3] <- 0.001
-    	upper[3] <- Inf
+    	lower <- c(0.0, 0.001, 0.001)
+    	upper <- rep(Inf, 3)
 
 	resop <- optim(param,fn,
 			   lower = lower,
@@ -674,20 +601,17 @@ bevholt <- function(rec, ssb, params = "missing") {
 	#  AIC calculation
 
 	.rhat <- resop$par[1]*ssb/(resop$par[2]+ssb)
-	logL <- 0
-	for(i in 1:length(rec)) {
-		logL <- logL + dnorm(log(rec[i]),log(.rhat[i]),sqrt(resop$par[3]),TRUE)
-	}
+	logL <- sum(dnorm(log(rec),log(.rhat),sqrt(resop$par[3]),TRUE))
 	.aic <- -2*logL+3
 
 	# return the results in the res list
 
-	res$rhat <- resop$par[1]*ssb/(resop$par[2]+ssb)
-	res$residuals <- log(rec)-log(res$rhat)
+	res$rechat <- resop$par[1]*ssb/(resop$par[2]+ssb)
+	res$residuals <- log(rec)-log(res$rechat)
 	res$params[,'alpha'] <- resop$par[1]
 	res$params[,'beta'] <- resop$par[2]
 	res$var <- resop$par[3]
-	acorr <- acf(res$residuals,plot=FALSE)$acf[2]
+	acorr <- acf(c(res$residuals),plot=FALSE)$acf[2]
 	res$varacorr <- resop$par[3]*(1-acorr*acorr)
 
 	# calculate covariance matrix from the Hessian:
@@ -718,8 +642,11 @@ segreg <- function(rec,ssb, params = "missing") {
 
 	# input checks
 	
-	if (!is.vector(rec) | !is.vector(ssb))
-		return(paste("Inputs not vectors"))
+	if(!is.FLQuant(rec) || !is.FLQuant(ssb)) stop("'rec' and 'ssb' must be 'FLQuant' objects")
+	
+	rec <- dimSums(rec)
+	ssb <- dimSums(ssb)
+
 
 	# create the output list
 
@@ -733,10 +660,6 @@ segreg <- function(rec,ssb, params = "missing") {
 		    varacorr=as.double(NA),
 		    aic=as.double(NA))
 
-	# create the predicted recruitment vector
-
-	.rhat <- vector(length=length(rec))
-
 	# perform the MLE using the optim() algorithm
 	# first, define the function to be optimised
 
@@ -744,64 +667,40 @@ segreg <- function(rec,ssb, params = "missing") {
 		.alpha <- x[1]
 		.beta <- x[2]
 		.sigma <- x[3]
-		for(i in 1:length(rec)) {
-			if(ssb[i] <= .beta)
-				.rhat[i] <- .alpha*ssb[i]
-			if(ssb[i] > .beta)
-				.rhat[i] <- .alpha*.beta
-		}
-		logL <- 0
-		for(i in 1:length(rec)) {
-			logL <- logL - dnorm(log(rec[i]),log(.rhat[i]),sqrt(.sigma),TRUE)
-		}
+
+        .rhat <- ifelse(c(ssb) <= .beta, .alpha*c(ssb), .alpha*.beta) 
+		return(- sum(dnorm(log(rec),log(.rhat),sqrt(.sigma),TRUE)))
 	}
+	
 
 	# setting up the initial guess and the bounds... 
 
 	param <- numeric(3)
-	lower <- numeric(3)
-	upper <- numeric(3)
 
 	if(!missing(params)) {
 		if(is.na(params[[1]]) | is.na(params[[2]])) { 
 			param[1] <- mean(rec/ssb)
 			param[2] <- mean(ssb)
-			for(i in 1:length(rec)) {
-				if(ssb[i] <= param[2])
-					.rhat[i] <- param[1]*ssb[i]
-				if(ssb[i] > param[2])
-					.rhat[i] <- param[1]*param[2]
-			}  
-			param[3] <- var(log(rec/.rhat))
+
+            .rhat <- ifelse(c(ssb) <= param[2], param[1]*c(ssb), param[1]*param[2])   
+			param[3] <- var(log(rec/.rhat), y= NULL, na.rm = TRUE)
 		} else {
 			param[1] <- params[[1]]
 			param[2] <- params[[2]]
-			for(i in 1:length(rec)) {
-				if(ssb[i] <= param[2])
-					.rhat[i] <- param[1]*ssb[i]
-				if(ssb[i] > param[2])
-					.rhat[i] <- param[1]*param[2]
-			} 
-			param[3] <- var(log(rec/.rhat))
+
+            .rhat <- ifelse(c(ssb) <= param[2], param[1]*c(ssb), param[1]*param[2]) 
+			param[3] <- var(log(rec/.rhat), y= NULL, na.rm = TRUE)
 		}
 	} else {
 		param[1] <- mean(rec/ssb)
 		param[2] <- mean(ssb)
-		for(i in 1:length(rec)) {
-			if(ssb[i] <= param[2])
-				.rhat[i] <- param[1]*ssb[i]
-			if(ssb[i] > param[2])
-				.rhat[i] <- param[1]*param[2]
-		}  
-		param[3] <- var(log(rec/.rhat)) 
+
+        .rhat <- ifelse(c(ssb) <= param[2], param[1]*c(ssb), param[1]*param[2])  
+		param[3] <- var(log(rec/.rhat), y= NULL, na.rm = TRUE)
 	}
 
-	lower[1] <- 0.001
-	upper[1] <- Inf
-	lower[2] <- 0.001
-	upper[2] <- Inf
-	lower[3] <- 0.001
-	upper[3] <- Inf
+	lower <- rep(0.001, 3)
+	upper <- rep(Inf, 3)
 	
 	resop <- optim(param,fn,
 			   lower = lower,
@@ -812,27 +711,20 @@ segreg <- function(rec,ssb, params = "missing") {
 	
 	# AIC calculation
 	
-	for(i in 1:length(rec)) {
-			if(ssb[i] <= resop$par[2]) 
-				.rhat[i] <- resop$par[1]*ssb[i]
-			if(ssb[i] > resop$par[2])
-				.rhat[i] <- resop$par[1]*resop$par[2]
-	}
-	logL <- 0
-	for(i in 1:length(rec)) {
-		logL <- logL + dnorm(log(rec[i]),log(.rhat[i]),sqrt(resop$par[3]),TRUE)
-	}
+    .rhat <- ifelse(c(ssb) <= resop$par[2], resop$par[1]*c(ssb), resop$par[1]*resop$par[2])
+	
+	logL <-  sum(dnorm(log(rec),log(.rhat),sqrt(resop$par[3]),TRUE))
 	.aic <- -2*logL+3
-
+	
 	# return the results in the res list
 
-	res$rechat <- .rhat
-	res$residuals <- log(rec)-log(res$rhat)
+	res$rechat <- FLQuant(.rhat, dimnames = dimnames(rec))
+	res$residuals <- log(rec)-log(res$rechat)
 	res$params[,'alpha'] <- resop$par[1]
 	res$params[,'beta'] <- resop$par[2] 
 	res$var <- resop$par[3]
-	acorr <- acf(res$residuals,plot=FALSE)$acf[2]
-	res$varacorr <- resop$par[3]*(1-acorr*acorr)
+	acorr <- acf(c(res$residuals),plot=FALSE)$acf[2]
+	res$varacorr <- resop$par[3]*(1-acorr^2)
 
 	# calculate covariance matrix from the Hessian:
 	# covar = inv(Hessian)/length(rec)
@@ -861,9 +753,11 @@ segreg <- function(rec,ssb, params = "missing") {
 qhstk <- function(rec, ssb, params = "missing") {
 
 	# input checks
+	if(!is.FLQuant(rec) || !is.FLQuant(ssb)) stop("'rec' and 'ssb' must be 'FLQuant' objects")
 	
-	if (!is.vector(rec) | !is.vector(ssb))
-		return(paste("Inputs not vectors"))
+	rec <- dimSums(rec)
+	ssb <- dimSums(ssb)
+
 
 	# create the output list
 
@@ -876,10 +770,6 @@ qhstk <- function(rec, ssb, params = "missing") {
 		    varacorr=as.double(NA),
 		    aic=as.double(NA))
 
-	# create the predicted recruitment vector
-
-	.rhat <- vector(length=length(rec))
-
 	# perform the MLE using the optim() algorithm
 
 	# first, define the function to be optimised
@@ -889,78 +779,56 @@ qhstk <- function(rec, ssb, params = "missing") {
 		.beta <- x[2]
 		.rho <- x[3]
 		.sigma <- x[4]
-		for(i in 1:length(rec)) {
-			if(ssb[i] <= .beta*(1-.rho)) 
-				.rhat[i] <- .alpha*ssb[i]
-			if(ssb[i] > .beta*(1-.rho) && ssb[i] < .beta*(1+.rho))
-				.rhat[i] <- .alpha*(ssb[i]-(ssb[i]-.beta*(1-.rho))*(ssb[i]-.beta*(1-.rho))/(as.double(4)*.rho*.beta))
-			if(ssb[i] >= .beta*(1+.rho))
-				.rhat[i] <- .alpha*.beta
-		}
-		logL <- 0
-		for(i in 1:length(rec)) {
-			logL <- logL - dnorm(log(rec[i]),log(.rhat[i]),sqrt(.sigma),TRUE)
-		}
+
+        .rhat <- ifelse(ssb <= .beta*(1-.rho), .alpha*ssb,
+                    ifelse(ssb < .beta*(1+.rho),
+                        .alpha*(ssb-(ssb-.beta*(1-.rho))*(ssb-.beta*(1-.rho))/(as.double(4)*.rho*.beta)),
+                        .alpha*.beta)) 
+		return(- sum(dnorm(log(rec),log(.rhat),sqrt(.sigma),TRUE)))
 	}
+	
 
 	# setting up the initial guess and the bounds... 
 
 	param <- numeric(4)
-	lower <- numeric(4)
-	upper <- numeric(4)
 
 	if(!missing(params)) {
 		if(is.na(params[[1]]) | is.na(params[[2]]) | is.na(params[[3]]) | missing(params)) {
 			param[1] <- mean(rec/ssb)
 			param[2] <- mean(ssb)
 			param[3] <- as.double(0.5)
-			for(i in 1:length(rec)) {
-				if(ssb[i] <= param[2]*(1-param[3])) 
-					.rhat[i] <- param[1]*ssb[i]
-				if(ssb[i] > param[2]*(1-param[3]) && ssb[i] < param[2]*(1+param[3]))
-					.rhat[i] <- param[1]*(ssb[i]-(ssb[i]-param[2]*(1-param[3]))*(ssb[i]-param[2]*(1-param[3]))/(as.double(4)*param[3]*param[2]))
-				if(ssb[i] >= param[2]*(1+param[3]))
-					.rhat[i] <- param[1]*param[2]
-			}
-			param[4] <- var(log(rec/.rhat))
+
+            .rhat <- ifelse(ssb <= param[2]*(1-param[3]), param[1]*ssb,
+                            ifelse(ssb <= param[2]*(1+param[3]), 
+                                param[1]*(ssb-(ssb-param[2]*(1-param[3]))*(ssb-param[2]*(1-param[3]))/(as.double(4)*param[3]*param[2])),
+                                param[1]*param[2]))         
+			param[4] <- var(log(rec/.rhat), y= NULL, na.rm = TRUE)
 		} else {
 			param[1] <- params[[1]]
 			param[2] <- params[[2]]
 			param[3] <- params[[3]]
-			for(i in 1:length(rec)) {
-				if(ssb[i] <= param[2]*(1-param[3])) 
-					.rhat[i] <- param[1]*ssb[i]
-				if(ssb[i] > param[2]*(1-param[3]) && ssb[i] < param[2]*(1+param[3]))
-					.rhat[i] <- param[1]*(ssb[i]-(ssb[i]-param[2]*(1-param[3]))*(ssb[i]-param[2]*(1-param[3]))/(as.double(4)*param[3]*param[2]))
-				if(ssb[i] >= param[2]*(1+param[3]))
-					.rhat[i] <- param[1]*param[2]
-			}
-			param[4] <- var(log(rec/.rhat)) 
+
+            .rhat <- ifelse(ssb <= param[2]*(1-param[3]), param[1]*ssb,
+                        ifelse(ssb < param[2]*(1+param[3]),
+                            param[1]*(ssb-(ssb-param[2]*(1-param[3]))*(ssb-param[2]*(1-param[3]))/(as.double(4)*param[3]*param[2])),
+                            param[1]*param[2])) 
+			param[4] <- var(log(rec/.rhat), y= NULL, na.rm = TRUE) 
 
 		}
 	} else {
 		param[1] <- mean(rec/ssb)
 		param[2] <- mean(ssb)
 		param[3] <- as.double(0.5)
-		for(i in 1:length(rec)) {
-			if(ssb[i] <= param[2]*(1-param[3])) 
-				.rhat[i] <- param[1]*ssb[i]
-			if(ssb[i] > param[2]*(1-param[3]) && ssb[i] < param[2]*(1+param[3]))
-				.rhat[i] <- param[1]*(ssb[i]-(ssb[i]-param[2]*(1-param[3]))*(ssb[i]-param[2]*(1-param[3]))/(as.double(4)*param[3]*param[2]))
-			if(ssb[i] >= param[2]*(1+param[3]))
-				.rhat[i] <- param[1]*param[2]
-		}
-		param[4] <- var(log(rec/.rhat))  
+
+		.rhat <- ifelse(ssb <= param[2]*(1-param[3]), param[1]*ssb,
+		              ifelse(ssb < param[2]*(1+param[3]),
+		                  param[1]*(ssb-(ssb-param[2]*(1-param[3]))*(ssb-param[2]*(1-param[3]))/(as.double(4)*param[3]*param[2])),
+                          param[1]*param[2]))    
+		param[4] <- var(log(rec/.rhat), y= NULL, na.rm = TRUE)  
 	}
 	
-	lower[1] <- 0.001
-	upper[1] <- Inf
-	lower[2] <- 0.001
-	upper[2] <- Inf
-	lower[3] <- 0.001
-	upper[3] <- 0.999
-	lower[4] <- 0.001
-	upper[4] <- Inf
+	lower <- rep(0.001, 4)
+	upper <- c(Inf, Inf, 0.999, Inf)
 	
 	resop <- optim(param,fn,
 			   lower = lower,
@@ -971,29 +839,24 @@ qhstk <- function(rec, ssb, params = "missing") {
 	
 	# AIC calculation
 	
-	for(i in 1:length(rec)) {
-			if(ssb[i] <= resop$par[2]*(1-resop$par[3])) 
-				.rhat[i] <- resop$par[1]*ssb[i]
-			if(ssb[i] > resop$par[2]*(1-resop$par[3]) && ssb[i] < resop$par[2]*(1+resop$par[3]))
-				.rhat[i] <- resop$par[1]*(ssb[i]-(ssb[i]-resop$par[2]*(1-resop$par[3]))*(ssb[i]-resop$par[2]*(1-resop$par[3]))/(as.double(4)*resop$par[3]*resop$par[2]))
-			if(ssb[i] >= resop$par[2]*(1+resop$par[3]))
-				.rhat[i] <- resop$par[1]*resop$par[2]
-	}
-	logL <- 0
-	for(i in 1:length(rec)) {
-		logL <- logL + dnorm(log(rec[i]),log(.rhat[i]),sqrt(resop$par[4]),TRUE)
-	}
+    .rhat <- ifelse(ssb <= resop$par[2]*(1-resop$par[3]), resop$par[1]*ssb,
+                ifelse(ssb < resop$par[2]*(1+resop$par[3]), 
+                    resop$par[1]*(ssb-(ssb-resop$par[2]*(1-resop$par[3]))*(ssb-resop$par[2]*(1-resop$par[3]))/(as.double(4)*resop$par[3]*resop$par[2])),        
+	                resop$par[1]*resop$par[2]))
+	                
+    logL <- sum(dnorm(log(rec),log(.rhat),sqrt(resop$par[4]),TRUE))
+	
 	.aic <- -2*logL+4
 
 	# return the results in the res list
 
-	res$rechat <- .rhat
-	res$residuals <- log(rec)-log(res$rhat)
+	res$rechat <- FLQuant(.rhat, dimnames = dimnames(rec))
+	res$residuals <- log(rec)-log(res$rechat)
 	res$params[,'alpha'] <- resop$par[1]
 	res$params[,'beta'] <- resop$par[2]
 	res$params[,'rho'] <- resop$par[3]
 	res$var <- resop$par[4]
-	acorr <- acf(res$residuals,plot=FALSE)$acf[2]
+	acorr <- acf(c(res$residuals),plot=FALSE)$acf[2]
 	res$varacorr <- resop$par[4]*(1-acorr*acorr)
 
 	# calculate covariance matrix from the Hessian:
@@ -1013,87 +876,40 @@ qhstk <- function(rec, ssb, params = "missing") {
 	return(res)
 } # }}}
 
-## sr()       {{{
+## sr()		{{{
 sr <- function(data, ...) {
-    
-    
-    sr <- data
-    model <- sr@model
-    params <- sr@params
-    mclist <- sr@mcmc
 
-    #slot(sr, "call") <- match.call()
-
-    # is model Bayesian or Evil?
-    #if(length(grep('Bayes', model)) != 0) {
-        #if(missing(data@mcmc) | !is.list(data@mcmc))
-            #stop("A Bayesian model needs a list of priors")
-    #}
-
-    # extract data
-    if (is.FLSR(data)) {
-        rec <- as.vector(data@rec)
-        ssb <- as.vector(data@ssb)
-    } else
-        stop("data must an FLSR object")
-    
-    # call selected model
-
-    if(length(grep('Bayes', model)) == 0) {
-    	call <- c(list(as.symbol(model), rec=rec, ssb=ssb, params))
+    # call non-Bayesian model
+    if(length(grep('Bayes', model(data))) == 0) {
+		# call("model", rec, ssb, params)	
+    	call <- c(list(as.symbol(model(data)), rec= rec(data), ssb = ssb(data), params(data)))
     	res <- try(eval(as.call(call)), silent=TRUE)
-	#res <- as.symbol(model)(rec,ssb)
-    } else {
-	call <- c(list(as.symbol(model), rec=rec, ssb=ssb, mclist))
- 	res <- try(eval(as.call(call)), silent=TRUE)
-    }
-    #print(res)
-
+	} else {
+	# call Bayesian models
+		call <- c(list(as.symbol(model(data)), rec=rec(data), ssb=ssb(data), mcmc(data)))
+	 	res <- try(eval(as.call(call)), silent=TRUE)
+	}
     # catch error
     if(inherits(res, "try-error")) {
         stop(res)
     }
 
-    # output for MLE fit
-    if(length(grep('Bayes', model)) == 0) {
-    	for(i in 1:length(res)) {
-        		if(is.FLQuant(slot(sr, names(res)[i]))) {
-				
-				# create temporary FLQuants from res and then 
-				# assign them to the relevant slots in sr object
-				
-				.units <- slot(sr, names(res)[i])@units
-				.nm  <- dimnames(slot(sr, names(res)[i]))
-				tmpflq <- FLQuant(quant="age",res[[i]],dim=c(1,length(res[[i]]),1,1,1),dimnames=.nm,units=.units)
-        			slot(sr, names(res)[i]) <- tmpflq
-			} else {
-            			try(slot(sr, names(res)[i]) <- res[[i]], silent=TRUE)
-			}
-    		}
-    }
-     
-    # ToDo: output for Bayes estimation
+	# list returned
+	if(is.list(res)) {
+		for (i in names(res))
+            slot(data, i) <- res[[i]]
+        
+        return(data)
+	}
+	# FLSR returned
+	else if(is.FLSR(res))
+		return(res)
+}	# }}}
 
-    if(length(grep('Bayes', model)) != 0) {
-
-    	# return Markov chains to relevant MCMC slots
-
-	sr@mcmc$params <- res
-
-	# now use the mean and covariance to fill out the other FLSR object slots: check just one chain
-	# ToDo
-    }
-	
-    return(sr)
-}    # }}}
-
-## Accesors
+## Accesors {{{
 # var accesor to overload var in stats
 if (!isGeneric("var")) {
-    setGeneric("var", function(x, y, na.rm, use){
-	    value <- standardGeneric("var")
-	    value
-    })
+	setGeneric("var", useAsDefault = var)
 }
 setMethod("var", signature(x="FLSR", y="missing", na.rm="missing", use="missing"),
 	function(x)
@@ -1113,4 +929,120 @@ setMethod("var<-", signature(object="FLSR", value="numeric"),
 )
 
 # automatic accesors, excluding var and range
-invisible(createFLAccesors(new("FLSR"), exclude=c('range', 'var')))
+invisible(createFLAccesors(new("FLSR"), exclude=c('range', 'var'))) # }}}
+
+
+FLSRstock <- function(object, rec.age){
+
+    validObject(object)
+
+    # recruitment delay set using minage
+    # from the FLStock object
+    if(missing(rec.age))
+        rec.age <- dims(stock.n(object))$min 
+    else{ 
+        if(rec.age < dims(stock.n(object))$min)
+            stop("Supplied recruitment age less than minimum age class")
+    }
+			     
+    if (all(is.na(slot(object, "stock.n"))) || all(is.na(slot(object, "stock.wt"))) ||
+        all(is.na(slot(object, "mat"))) || all(is.na(slot(object, "harvest.spwn"))) || 
+	    all(is.na(slot(object, "m.spwn"))) || all(is.na(slot(object, "harvest"))) || 
+	    all(is.na(slot(object, "m")))) 
+        stop("stock must have 'stock.n', 'stock.wt', 'harvest', 'harvest.spwn', 
+            'm', 'm.spwn' and 'mat'")
+
+    # calculate ssb and create FLSR object incorprating rec.age
+        
+    rec <- dimSums(object@stock.n[as.character(rec.age),])
+	    
+   	if(units(slot(object,"harvest")) != 'f' && units(slot(object,"harvest")) != 'hr')
+            stop("Incorrect units specified in harvest slot of FLStock object")
+        
+    if(units(slot(object, "harvest")) == 'f')
+        ssb <- apply(slot(object, "stock.n") * exp(-harvest(object, "f") *
+                slot(object,"harvest.spwn") - slot(object, "m")*
+                slot(object, "m.spwn"))*slot(object, "stock.wt")*slot(object, "mat"), 2:5, sum)
+    else    # harvest units == 'hr' 
+		ssb <- apply(slot(object, "stock.n") * (1 - slot(object, "harvest") * slot(object, "harvest.spwn")) *
+            exp(-slot(object, "m") * slot(object, "m.spwn")) * slot(object, "mat") * slot(object, "stock.wt"), 2:5, sum)	
+
+    # now alter the stock and recruitment 
+    # vectors to factor in the recruitement age
+	
+    if((dim(rec)[2]-1) <= rec.age)
+        stop("FLStock recruitment data set too short")
+		
+    rec <- rec[,(1+rec.age):dim(rec)[2],,,]
+    ssb <- ssb[,1:(dim(ssb)[2] - rec.age),,,]
+
+	# create the FLSR object
+
+	sr <- new("FLSR", rec = rec,ssb = ssb) 
+    
+    units(slot(sr, "rec")) <- units(slot(object, "stock.n"))
+	units(slot(sr, "ssb")) <- units(slot(object, "stock.wt")) 
+
+#    slot(sr, "rechat") <- FLQuant(dimnames = dimnames(slot(sr, "rec")))
+    units(slot(sr, "rechat")) <- units(slot(sr, "rec"))
+        
+#    slot(sr, "residuals") <- FLQuant(dimnames = dimnames(slot(sr, "rec")))
+#    slot(sr, "name") <- name
+#    slot(sr, "desc") <- desc
+#    slot(sr, "model") <- model
+    
+    return(sr)
+
+}
+
+FLSRbiol <- function(object, rec.age){
+
+    validObject(object)
+
+    # recruitment delay set using minage
+    # from the FLStock object
+    if(missing(rec.age))
+        rec.age <- dims(n(object))$min 
+    else{ 
+        if(rec.age < dims(n(object))$min)
+            stop("Supplied recruitment age less than minimum age class")
+    }
+			     
+    if (all(is.na(slot(object, "n"))) || all(is.na(slot(object, "wt"))) ||
+        all(is.na(slot(object, "fec"))) || all(is.na(slot(object, "spwn"))))
+         
+        stop("biol must have 'n', 'wt', 'm', 'fec' and 'spwn'")
+
+    # calculate ssb and create FLSR object incorprating rec.age
+        
+    rec <- dimSums(object@n[as.character(rec.age),])
+	            
+    ssb <- apply(slot(object, "n") * exp(- slot(object, "m")*slot(object, "spwn"))*
+        slot(object, "wt")*slot(object, "fec"), 2:5, sum)
+   
+    # now alter the stock and recruitment 
+    # vectors to factor in the recruitement age
+	
+    if((dim(rec)[2]-1) <= rec.age)
+        stop("FLBiol recruitment data set too short")
+		
+    rec <- rec[,(1+rec.age):dim(rec)[2],,,]
+    ssb <- ssb[,1:(dim(ssb)[2] - rec.age),,,]
+
+	# create the FLSR object
+
+	sr <- new("FLSR", rec = rec,ssb = ssb) 
+    
+    units(slot(sr, "rec")) <- units(slot(object, "n"))
+	units(slot(sr, "ssb")) <- units(slot(object, "wt")) 
+
+#    slot(sr, "rechat") <- FLQuant(dimnames = dimnames(slot(sr, "rec")))
+    units(slot(sr, "rechat")) <- units(slot(sr, "rec"))
+        
+    slot(sr, "residuals") <- FLQuant(dimnames = dimnames(slot(sr, "rec")))
+#    slot(sr, "name") <- name
+#    slot(sr, "desc") <- desc
+#    slot(sr, "model") <- model
+    
+    return(sr)
+}
